'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['original/dashboard-core/js/dashboard/layout/views/Tab', 'react', 'react-dom', 'corsair_ship/ba-react-html5-dnd', 'ca-ui-toolkit', 'dashboard-core/js/lib/@waca/core-client/js/core-client/utils/BidiUtil', 'underscore', 'jquery'], function (Tab, React, ReactDOM, ReactHtml5DnDContext, UI_Toolkit, BidiUtil, _, $) {
	var TabWidget = function (_Tab) {
		_inherits(TabWidget, _Tab);

		function TabWidget() {
			_classCallCheck(this, TabWidget);

			return _possibleConstructorReturn(this, _Tab.apply(this, arguments));
		}

		TabWidget.prototype._registerEvents = function _registerEvents() {
			_Tab.prototype._registerEvents.call(this);
			this.model.on('change:showDropDown', this._updateProperties, this);
		};

		TabWidget.prototype._unregisterEvents = function _unregisterEvents() {
			_Tab.prototype._unregisterEvents.call(this);
			this.model.off('change:showDropDown', this._updateProperties, this);
		};

		TabWidget.prototype.onSelection = function onSelection(event) {
			var _this3 = this;

			var idList = event.info.value;
			if (event.name === 'change:selections:select' && !idList.find(function (id) {
				return _this3.model.id === id || _this3.model.items.find(function (item) {
					return item.getId() === id;
				});
			})) {
				return;
			}

			event.info.value.forEach(function (id) {
				var canvas = _this3.canvas;
				var content = canvas.getContent(id);
				if (content && content.getType() === 'page') {
					if (_this3.selectedModel && _this3.selectedModel.id !== id) {
						canvas.deselectContent([_this3.selectedModel.id]);
					}
					_this3._TabsWidget && _this3._TabsWidget.setState({ selectedTab: _this3._getPanelId(id) });
				}
			});
		};

		TabWidget.prototype._createTabPanel = function _createTabPanel(pageModel, pageContent, tabColorProperties) {
			var _this4 = this;

			var TabPanel = UI_Toolkit.TabPanel;
			var _tabIconObj = pageModel.get('tabIcon');
			var tabIcon = _tabIconObj && _tabIconObj !== 'transparent' ? this._icons.getIcon(_tabIconObj).id : null;

			var properties = {
				id: this._getPanelId(pageModel.id),
				title: pageModel.get('title'),
				label: pageModel.get('title'),
				icon: tabIcon,
				keepTabContent: true,
				tooltipOrient: 'top'
			};

			_.extend(properties, tabColorProperties);

			return React.createElement(TabPanel, properties, [React.createElement('div', { ref: function ref(node) {
					if (node) {
						$(node).append(pageContent);
					} else {
						// Handle child tab widget. Use children() instead of find()
						//this.$el.find('.tabPageHolder').append(pageContent);
						_this4.$el.children('.tabPageHolder').append(pageContent);
					}
				} })]);
		};

		TabWidget.prototype._renderTabs = function _renderTabs(selectedId) {
			var _this5 = this;

			// If we this is not a tab level tab, tag the element with the class that will enale to behave like a wieget
			// This are currently defined in the fragment, but there is a bug in the html templates that will clear them when the fill color is set on the tabb widget
			if (this.content.getContainer()) {
				// widget class will enable the move resizer
				// forcePageFit will force the tab content to resize with the tab control resizes

				this.$el.addClass("widget forcePageFit");
			}

			// widgetContent  is used with userPrimaryFGPaletteColor to enable the auto foreground colors
			this.$el.find('.tabCntr').addClass('widgetContent');

			var Tabs = UI_Toolkit.Tabs;
			var panels = [],
			    panelsMirror = []; //Mirror entry of the tabs that we can compare with for tab changes.

			this.model.items.forEach(function (item) {
				var colorProperties = _this5._getColorProperties(item);
				var pageNode = _this5.$el.find('#' + item.id + '_tab');
				var tabPanel = _this5._createTabPanel(item, pageNode, colorProperties);
				panels.push(tabPanel);
				panelsMirror.push(tabPanel);
			});

			var tabPosition = this.model.get('tabPosition');
			var tabIconPosition = this.model.get('tabIconPosition');
			var _this = this;
			var content = _this.canvas.getContent(this.model.id);
			var isDropDownHeader = content && content.getPropertyValue('showDropDown');

			var TabsWidget = function (_React$Component) {
				_inherits(TabsWidget, _React$Component);

				function TabsWidget(props) {
					_classCallCheck(this, TabsWidget);

					var _this6 = _possibleConstructorReturn(this, _React$Component.call(this, props));

					_this6.state = {
						selectedTab: _this._getPanelId(selectedId),
						tabs: panels,
						editable: false,
						vertical: true,
						draggable: false,
						addable: false,
						deleteable: false,
						placement: tabPosition || 'top',
						linePosition: _this._getDefaultLinePostion(tabPosition),
						tabIconPosition: tabIconPosition,
						isDropDownHeader: isDropDownHeader
					};
					_this6._tabsMirror = panelsMirror;
					_this6._icons = _this._icons;
					_this6.tabListNode = null;
					_this6.toolBarElementsList = [];
					return _this6;
				}

				TabsWidget.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
					if (this._updatedTabs) {
						this._updatedTabs = false;
						if (this.state.tabs.length < this._tabsMirror.length) {
							//Delete
							_this.deleteView(_this._getModelIdFromPanel(_.difference(this._tabsMirror, this.state.tabs)[0].props.id));
						}
					}
					this._tabsMirror = this.state.tabs.slice();
					if (prevState.selectedTab !== this.state.selectedTab) {
						_this.showView(_this.model.findModel(_this._getModelIdFromPanel(this.state.selectedTab)));
					}
				};

				TabsWidget.prototype.componentDidMount = function componentDidMount() {
					// Handle child tab widget - use first() to scope to the proper tab list
					//const tabList = _this.$el.find('.ba-common-tabList');
					var tabList = _this.$el.find('.ba-common-tabList').first();
					tabList.on('click.dbTab', _this._tabClick.bind(_this));
					tabList.on('keydown.dbTab', _this._tabKeydown.bind(_this));
					tabList.on('dblclick.dbTab', _this._tabDoubleClick.bind(_this));
					this.tabListNode = tabList;
				};

				TabsWidget.prototype.componentWillUnmount = function componentWillUnmount() {
					// Handle child tab widget - use first() to scope to the proper tab list
					//const tabList = _this.$el.find('.ba-common-tabList');
					var tabList = _this.$el.find('.ba-common-tabList').first();
					tabList.off('click.dbTab');
					tabList.off('keydown.dbTab');
					tabList.off('dblckick.dbTab');
					this.tabListNode = null;
					this.hasUnmounted = true;
				};

				TabsWidget.prototype.updateTheme = function updateTheme() {
					var newTabs = [];
					for (var i = 0; i < this.state.tabs.length; i++) {
						var pageModel = _this.model.items[i];
						var colorProperties = _this._getColorProperties(pageModel);
						newTabs.push(React.cloneElement(this.state.tabs[i], colorProperties));
					}
					this.setState({ tabs: newTabs });
				};

				TabsWidget.prototype.editTabLabel = function editTabLabel(startEdit, model) {
					var _this7 = this;

					var isEditable = !!startEdit;
					var $node = void 0;
					if (this.tabListNode) {
						$node = $(this.tabListNode);
					} else {
						// Handle child tab widget - use first() to scope to the proper tab list
						//$node = _this.$el.find('.ba-common-tabList');
						$node = _this.$el.find('.ba-common-tabList').first();
					}
					$node.find('[role=tab][aria-selected=true]').focus(); //HACK - Move the focus to the tab. If the focus happens to be on the label when editing is turned on, the re-render may happen before we try to focus.

					this.setState({ editable: isEditable }, function () {
						if (isEditable) {
							//let label = $node.find('[contentEditable=true]');
							var label = $node.find('[contentEditable=true]');
							var panelId = _this._getPanelId(model.id);
							var originalLabel = void 0;
							var originalTab = _.find(_this7.state.tabs, function (panel) {
								return panelId === panel.props.id;
							});
							if (originalTab) {
								originalLabel = originalTab.props.label;
							}
							var observer = new MutationObserver(function () {
								//HACK HACK HACK HACK ------- Currently, on the first focus to the editable area, the toolkit tabs are re-rendering, thus blurring the focus immediately. The field needs to be found again and re-focused after the re-render (As there is a new label after this) to actually edit.
								if (!$.contains(_this.$el, label)) {
									observer.disconnect(); //The re-render has happened, so we no longer need this observer.
									label = $node.find('[contentEditable=true]');

									// TEMPORARY CODE - Replace when the tabs provide a callback when editing is complete
									label.on('blur.dashboardtabs', function () {
										label.off('blur.dashboardtabs');
										_this7.setState({ editable: false }, function () {
											var newTab = _.find(_this7.state.tabs, function (panel) {
												return panelId === panel.props.id;
											});
											if (newTab) {
												var newLabel = label.text();
												if (originalLabel !== newLabel) {
													newTab.props.title = newLabel;
													_this.updateViewLabel(model, newLabel);
												}
											}
										});
									});

									label.focus();
								}
							});
							observer.observe(_this.$el[0], { childList: true, subtree: true }); //Observe changes in descendant nodes of the tabs to catch the re-render caused by the first focus.

							label.focus();
						}
					});
				};

				TabsWidget.prototype.updateTabLabel = function updateTabLabel(model) {
					var newTabs = [];
					var panelId = _this._getPanelId(model.id);
					for (var i = 0; i < this.state.tabs.length; i++) {
						var tab = this.state.tabs[i];
						var title = model.get('title');
						if (tab.props.id == panelId && tab.props.label !== title) {
							newTabs.push(React.cloneElement(tab, {
								label: title,
								title: title
							}));
						} else {
							newTabs.push(tab);
						}
					}
					var old = JSON.stringify(this.state.tabs);
					if (old !== JSON.stringify(newTabs)) {
						this.setState({ tabs: newTabs });
					}
				};

				TabsWidget.prototype.updateTabIcon = function updateTabIcon(model) {
					var newTabs = [],
					    changeMade = false;
					var panelId = _this._getPanelId(model.id);
					for (var i = 0; i < this.state.tabs.length; i++) {
						var tab = this.state.tabs[i];
						var _tabIconObj = model.get('tabIcon');

						var icon = void 0;
						if (_tabIconObj && _tabIconObj !== 'transparent') {
							icon = this._icons.getIcon(_tabIconObj).id;
						} else {
							icon = null;
						}

						if (tab.props.id === panelId && tab.props.icon !== icon && !this.translationMode) {
							newTabs.push(React.cloneElement(tab, {
								icon: icon
							}));
							changeMade = true;
						} else {
							newTabs.push(tab);
						}
					}
					if (changeMade) {
						this.setState({ tabs: newTabs });
					}
				};

				/**
     * @param  {Object} model: the model of a view whose children element need to be translated
     * @param  {Boolean} shown: when it is true means we want to show the translate icon for this model.
     * @returns {Int} numberOfChanges: the number of tabs we've updated.
     */


				TabsWidget.prototype.updateTranslationIcon = function updateTranslationIcon(model, shown) {
					var icons = _this.dashboardApi.getFeature('Icons');
					this.translationMode = shown;
					var newTabs = [],
					    changeMade = false,
					    numberOfChanges = 0;
					var panelId = _this._getPanelId(model.id);
					for (var i = 0; i < this.state.tabs.length; i++) {
						var tab = this.state.tabs[i];
						if (tab.props.id == panelId) {
							if (shown) {
								newTabs.push(React.cloneElement(tab, {
									icon: icons.getIcon('warning').id,
									className: 'translateIcon'
								}));
								changeMade = true;
								numberOfChanges++;
							} else if (tab.props.icon) {
								var tabIcon = model.get('tabIcon');
								newTabs.push(React.cloneElement(tab, {
									icon: tabIcon ? tabIcon.name : undefined,
									className: undefined
								}));
								changeMade = true;
								numberOfChanges++;
							}
						} else {
							newTabs.push(tab);
						}
					}
					if (changeMade) {
						this.setState({ tabs: newTabs });
					}
					return numberOfChanges;
				};

				TabsWidget.prototype.render = function render() {
					var _this8 = this;

					var tabColorProperties = _this._getColorProperties();
					var tabs = this.state.tabs;
					tabs = tabs.map(function (tab) {
						return React.cloneElement(tab, {
							iconPosition: _this8.state.tabIconPosition
						});
					});

					_this._TabsWidget = this;
					return React.createElement('div', {
						className: 'ba-theme-default userPrimaryFGPaletteColor',
						style: {}
					}, [React.createElement(UI_Toolkit.Select, {
						selected: _this.selectedItem || _this.model.items[0] && _this.model.items[0].get('id'),
						options: _this.model.items.map(function (item) {
							return {
								value: item.get('id'),
								label: item.get('title')
							};
						}),
						variant: 'frameless',
						onChange: function onChange(value) {
							_this.selectedItem = value;
							_this.onSelection({ info: { value: [value] } });
						}
					}), React.createElement(Tabs, {
						draggable: this.state.draggable,
						selected: this.state.selectedTab,
						stateTabs: this.state.tabs,
						editable: this.state.editable,
						deleteable: this.state.deleteable,
						addable: this.state.addable,
						textDir: BidiUtil.userPreferredTextDir,
						onChange: function onChange(tabId) {
							var newTabId = _this._getModelIdFromPanel(tabId);
							_this.canvas.selectContent([newTabId]);
						},
						setState: function setState(option) {
							_this8._updatedTabs = true;_this8.setState({ tabs: option });
						},
						onAddButtonClick: function onAddButtonClick() {
							return _this.onAddClick();
						},
						onReorder: function onReorder(newIndex, tabId) {
							_this.onReorder(tabs, newIndex, tabId);
						},
						scrollButtonsBackgroundColor: _this.colorsService.getLayoutBackgroundColorHex(_this.model.get('fillColor')),
						scrollButtonsTextColor: tabColorProperties.textColor,
						onDeletePrompt: _this.onDeletePrompt,
						placement: this.state.placement,
						linePosition: this.state.linePosition
					}, tabs)]);
				};

				return TabsWidget;
			}(React.Component);

			var DragEnabledTabsWidget = ReactHtml5DnDContext(TabsWidget);
			ReactDOM.render(React.createElement(DragEnabledTabsWidget), this.$el.find('.tabCntr')[0]);
		};

		TabWidget.prototype.showView = function showView(model) {
			var sTabId = this._getViewId(model);
			if (this._currentViewId === sTabId) {
				return;
			}

			if (this.selectedModel) {
				var layoutView = this.layoutController.getLayoutView(this.selectedModel.id);
				if (layoutView) {
					layoutView.onHide();
				}
			}

			// Handle child tab widget - use first() to scope to the proper tab list
			//this.$el.find('.pageTabContent.selected').removeClass('selected');
			this.$el.find('.pageTabContent.selected').first().removeClass('selected');
			$('#' + sTabId, this.$el).addClass('selected');

			this._currentViewId = sTabId;
			this.selectedModel = model;

			this._TabsWidget.setState({ selectedTab: this._getPanelId(this.selectedModel.id) });
			this.eventRouter.trigger('tab:tabChanged', { modelId: this.selectedModel.id, isTabWidget: !!this.model.getParent() });

			return this.layoutController.getLayoutViewWhenReady(this.selectedModel.id).then(function (layoutView) {
				layoutView.onShow();
				return this._invokeLifeCycleHandlers('tabs.onShow', {
					$el: this.$el,
					model: model
				}).then(function () {
					//default behavior goes in here.
					this._updateHeader();
					//	this.$el.css('background-color', 'white');
				}.bind(this));
			}.bind(this));
		};

		TabWidget.prototype._updateHeader = function _updateHeader() {
			var tabTitleElements = this.$el.find('.ba-common-select__wrapper');
			if (!this.model.getParent()) {
				this.$el.removeClass('tabsHidden');
				_.each(tabTitleElements, function (tabTitle) {
					if (!$(tabTitle).closest('.page.pagetab.widget.forcePageFit').length) {
						if (tabTitle && tabTitle.children && tabTitle.children.length && !tabTitle.children[0].classList.contains('dropDownWidget')) {
							$(tabTitle).addClass('hide');
						}
					}
				});
			} else if (!this.model.get('showDropDown')) {
				this.$el.removeClass('tabsHidden');
				_.each(tabTitleElements, function (tabTitle) {
					if (tabTitle && tabTitle.children && tabTitle.children.length && !tabTitle.children[0].classList.contains('dropDownWidget')) {
						$(tabTitle).addClass('hide');
					}
				});
			} else {
				this.$el.addClass('tabsHidden');
				tabTitleElements.removeClass('hide');
			}
		};

		TabWidget.prototype._updateProperties = function _updateProperties() {
			this._updateHeader();
			this.eventRouter.trigger('properties:refreshPane');
		};

		TabWidget.prototype._setTabsVisibility = function _setTabsVisibility() {
			var hideTabs = this.model.get('showDropDown') || !this.content.getPropertyValue('showTabsInPreview') && !this._dashboardState.getUiState().authoring;
			this.$el.toggleClass('tabsHidden', hideTabs);
		};

		/* Fetch the TabPane object given the TabList tab node.
   */


		TabWidget.prototype._getPanelFromNode = function _getPanelFromNode(tabNode) {
			if (tabNode) {
				// Handle child tab widget - use first() to scope to the proper tab list
				//const tabIndex = this.$el.find('.ba-common-tabList [role=tab]').index(tabNode);
				var tabIndex = this.$el.find('.ba-common-tabList').first().find('[role=tab]').index(tabNode);

				if (tabIndex > -1) {
					return this._TabsWidget.state.tabs[tabIndex];
				}
			}
			return null;
		};

		return TabWidget;
	}(Tab);

	return TabWidget;
});
//# sourceMappingURL=TabWidget.js.map