'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Licensed Materials - Property of IBM
 * IBM Cognos Products: BI Cloud (C) Copyright IBM Corp. 2020
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
define(['original/dashboard-core/js/dashboard/model/LayoutModel'], function (LayoutModel) {
	var TabWidgetLayoutModel = function (_LayoutModel) {
		_inherits(TabWidgetLayoutModel, _LayoutModel);

		function TabWidgetLayoutModel(spec, boardModel, logger) {
			_classCallCheck(this, TabWidgetLayoutModel);

			return _possibleConstructorReturn(this, _LayoutModel.call(this, spec, boardModel, logger));
		}

		TabWidgetLayoutModel.prototype.init = function init(spec, boardModel, logger) {
			TabWidgetLayoutModel.inherited('init', this, arguments);
			this.boardModel = boardModel;
			this.whitelistAttrs = ['id', 'from', 'css', 'items', 'style', 'type', 'title', 'templateName', 'relatedLayouts', 'clones', 'fillColor', 'disableScrollDrop', 'showGrid', 'snapGrid', 'snapObjects', 'tabTextColor', 'tabSelectedLineColor', 'tabBackgroundColor', 'pageSize', 'hideTab', 'showDropDown', 'tabPosition', 'tabIconPosition', 'tabIcon', 'tabIconColor', 'layoutPositioning', 'fitPage', 'content'];
			this.colorProperties = ['fillColor', 'tabTextColor', 'tabSelectedLineColor', 'tabBackgroundColor', 'tabIconColor'];

			// eslint-disable-next-line no-undef
			this.selected = new Set();

			if (this.boardModel.layoutExtensions) {
				var modelExtensions = this.boardModel.layoutExtensions;
				Object.keys(modelExtensions || {}).forEach(function (property) {
					this.whitelistAttrs.push(property);
					if (modelExtensions[property] && typeof modelExtensions[property].init === 'function') {
						modelExtensions[property].init(this, spec, boardModel, logger);
					}
				}, this);
			}

			// Notify the board model when there is a layout change to the items so that the event is propagated to the auto save and the undo/redo controller
			this.on('op', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:title', this.boardModel.onLayoutChange, this.boardModel);
			// TODO: for change:css, we should move away from monitoring/setting the css property in the model and set specify properties
			this.on('change:css', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:fillColor', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:type', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:showGrid', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:snapGrid', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:snapObjects', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabTextColor', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabSelectedLineColor', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabBackgroundColor', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:pageSize', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:hideTab', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabPosition', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabIconPosition', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabIcon', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:tabIconColor', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:layoutPositioning', this.boardModel.onLayoutChange, this.boardModel);
			this.on('change:fitPage', this.boardModel.onLayoutChange, this.boardModel);

			this._initializeContentModel(this.content);

			this.logger = logger;

			var idMap = {};
			if (spec.items) {
				this.items = [];
				for (var i = 0, iLen = spec.items.length; i < iLen; i++) {
					var item = new TabWidgetLayoutModel(spec.items[i], this.boardModel);
					if (!idMap[item.id]) {
						this.items.push(item);
						idMap[item.id] = true;
					} else {
						this.logger.error('Found duplicate layout id "' + item.id + '" on initialization of layout "' + this.id + '".');
					}
				}
			}

			return this;
		};

		/**
   * Determine the layout positioning for the given model.
   * Walk up the parent tree until found.
   * @param {boolean} includeParents - Search parents if not defined on this model
   * @return {string} Either 'absolute' or 'relative'
   */


		TabWidgetLayoutModel.prototype.getLayoutPositioning = function getLayoutPositioning(includeParents) {
			if (this.type === 'genericPage') {
				if (this._hasInnerTabParent()) {
					return;
				}
				return this.layoutPositioning || 'relative';
			} else if (includeParents) {
				var parent = this.getParent();
				if (parent) {
					return parent.getLayoutPositioning();
				}
				return undefined;
			} else {
				return undefined;
			}
		};

		TabWidgetLayoutModel.prototype._hasInnerTabParent = function _hasInnerTabParent() {
			var tabParent = this.getParent && this.getParent();
			while (tabParent && tabParent.type !== 'tab') {
				tabParent = tabParent.getParent && tabParent.getParent();
			}
			return !!(tabParent && tabParent.getParent && tabParent.getParent());
		};

		return TabWidgetLayoutModel;
	}(LayoutModel);

	return TabWidgetLayoutModel;
});
//# sourceMappingURL=TabWidgetLayoutModel.js.map